//---------------------------------------------------------------------------
#pragma hdrstop

#include "PICFunctions.h"       //       PIC
#include "PortFunctions.h"      //      
#include "LogFunctions.h"       //      
#include "ChipFunctions.h"      //     
#include "Main.h"               //      
//---------------------------------------------------------------------------
#pragma package(smart_init)
////////////////////////////////////////////////////////////
//      ,     //
////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------
//---    C ( B)    ---------------------
//---------------------------------------------------------------------------
bool SetInitState(void)
{ WRITE_BUFFER wr_buf;          //    

  //--- all outputs = 0; all inputs without pull-ups; Clock, DataOut = outputs, all others = inputs ---
  wr_buf.Size=4;
  wr_buf.Data[0]=0x11;                  // PORTCSetup
  wr_buf.Data[1]=0x0;                   //   ,    
  wr_buf.Data[2]=0x0F;                  // DDRCSetup
  wr_buf.Data[3]=0x30;                  // 0011 0000: Clock, DataOut - 
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set InitState");      //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //   PGM    PB0  ,       1
  wr_buf.Size=4;
  wr_buf.Data[0]=0x0D;                  // PORTBSetup
  wr_buf.Data[1]=0x0;                   //   ,    
  wr_buf.Data[2]=0x0B;                  // DDRBSetup
  wr_buf.Data[3]=0x01;                  // 0000 0001: PB0 - 
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," set InitState");      //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  return true;
}
//---------------------------------------------------------------------------
//---    ,  VPP-first ------------------
//---------------------------------------------------------------------------
bool EnterHVPVMode_VPPfirst(void)
{ WRITE_BUFFER wr_buf;          //    

  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;          // DoExtendedCommand
  wr_buf.Data[1]=0x04;          // ExecuteVppFirstAlg_PIC (       )
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," enter programming mode, Vpp-first");  //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  return true;
}
//---------------------------------------------------------------------------
//---    ,  VDD-first ------------------
//---------------------------------------------------------------------------
bool EnterHVPVMode_VDDfirst(void)
{ WRITE_BUFFER wr_buf;          //    

  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;          // DoExtendedCommand
  wr_buf.Data[1]=0x05;          // ExecuteVddFirstAlg_PIC (       )
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," enter programming mode, Vdd-first");  //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  return true;
}
//---------------------------------------------------------------------------
//---    ,  VPP-last -------------------
//---------------------------------------------------------------------------
bool ExitHVPVMode_VPPlast(void)
{ WRITE_BUFFER wr_buf;          //    

  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;          // DoExtendedCommand
  wr_buf.Data[1]=0x06;          // ExecuteVppLastAlg_PIC (       )
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," exit programming mode, Vpp-last");    //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  return true;
}
//---------------------------------------------------------------------------
//---    ,  VDD-last -------------------
//---------------------------------------------------------------------------
bool ExitHVPVMode_VDDlast(void)
{ WRITE_BUFFER wr_buf;          //    

  wr_buf.Size=2;
  wr_buf.Data[0]=0x1D;          // DoExtendedCommand
  wr_buf.Data[1]=0x07;          // ExecuteVddLastAlg_PIC (       )
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," exit programming mode, Vdd-last");    //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  return true;
}
//---------------------------------------------------------------------------
//---    -----------------------------------------------
//---------------------------------------------------------------------------
unsigned int ReadSignatureBytes(void)
{ WRITE_BUFFER wr_buf;          //    
  READ_BUFFER rd_buf;           //    
  unsigned int ChipID;          //  

  //    6  (  )
  wr_buf.Size=2;
  wr_buf.Data[0]=0x42;          // LoadPaketSize_SPI
  wr_buf.Data[1]=0x06;          //  
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," LoadPaketSize_SPI = 0x6");    //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //   Load Configuration (    )
  wr_buf.Size=2;
  wr_buf.Data[0]=0x4C;          // FastSendPaket_SPI
  wr_buf.Data[1]=0x00;          // Load Configuration
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," Load Configuration");         //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //    16  ( / )
  wr_buf.Size=2;
  wr_buf.Data[0]=0x42;          // LoadPaketSize_SPI
  wr_buf.Data[1]=0x10;          //  
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," LoadPaketSize_SPI = 0x10");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //         Load Configuration
  //     ,       Begin Programming
  wr_buf.Size=3;
  wr_buf.Data[0]=0x4C;          // FastSendPaket_SPI
  wr_buf.Data[1]=0x00;          // dummy byte
  wr_buf.Data[2]=0x00;          // dummy byte
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," FastSendPaket_SPI 0x00 0x00");    //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //   6    Increment Address     0x2006 (   ID)
  //    6  (  )
  wr_buf.Size=2;
  wr_buf.Data[0]=0x42;          // LoadPaketSize_SPI
  wr_buf.Data[1]=0x06;          //  
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," LoadaketSize_SPI = 0x06");    //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  for(unsigned char i=0; i<6; i++)
  { //   Increment Address
    wr_buf.Size=2;
    wr_buf.Data[0]=0x4C;          // FastSendPaket_SPI
    wr_buf.Data[1]=0x06;          // Increment Address
    if(!WriteTo(wr_buf)) { SendMsgToLog(6," Increment Address");        //   
                           CloseHandle(Port);
                           Port=INVALID_HANDLE_VALUE;
                           return 0;
                         }
    Sleep(1);
    Sleep(Delays);//////////
  }
  //   Read Data From Program Memory
  wr_buf.Size=2;
  wr_buf.Data[0]=0x4C;          // FastSendPaket_SPI
  wr_buf.Data[1]=0x04;          // Read Data From Program Memory
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," Read Data From Program Memory");      //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //    16  ( / )
  wr_buf.Size=2;
  wr_buf.Data[0]=0x42;          // LoadPaketSize_SPI
  wr_buf.Data[1]=0x10;          //  
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," LoadPaketSize_SPI = 0x10");   //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //      (       ,  -  -   )
  wr_buf.Size=3;
  wr_buf.Data[0]=0x4C;          // FastSendPaket_SPI
  wr_buf.Data[1]=0xFE;          // dummy byte with Start bit
  wr_buf.Data[2]=0x7F;          // dummy byte with Stop bit
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," FastSendPaket_SPI 0xFE 0x7F");        //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //     (14    -     )
  wr_buf.Size=1;
  wr_buf.Data[0]=0x4A;          // ReadSignPaket_SPI
  if(!WriteTo(wr_buf)) { SendMsgToLog(6," ReadSignPaket_SPI");  //   
                         CloseHandle(Port);
                         Port=INVALID_HANDLE_VALUE;
                         return 0;
                       }
  Sleep(1);
  Sleep(Delays);//////////
  //   
  rd_buf=ReadFrom(2);                                   // 
  if(!rd_buf.Size) { SendMsgToLog(7," from RH-0010");   //   
                     CloseHandle(Port);
                     Port=INVALID_HANDLE_VALUE;
                     return 0;
                   }
  //---  ID 
  ChipID = (rd_buf.Data[0] + (rd_buf.Data[1] << 8)) >> 1;       //           1 
  return ChipID;
}
//---------------------------------------------------------------------------

